
#include <io_jpeg.hpp>
#include <io_exception.hpp>

#include <iostream>
#include <sstream>

#include <jpeglib.h>
#include <fstream>

namespace cpe
{

image io_jpeg::read(const std::string& filename) const
{
    unsigned char r,g,b;
    int width, height;
    struct jpeg_decompress_struct cinfo;
    struct jpeg_error_mgr jerr;

    FILE * infile;        /* source file */
    JSAMPARRAY pJpegBuffer;       /* Output row buffer */
    int row_stride;       /* physical row width in output buffer */
    if ((infile = fopen(filename.c_str(), "rb")) == NULL)
    {
        std::stringstream msg;msg<<"Error loading ";msg<<filename;
        throw cpe::io_exception(msg.str(),__FILE__,__FUNCTION__,__LINE__);
    }
    cinfo.err = jpeg_std_error(&jerr);
    jpeg_create_decompress(&cinfo);
    jpeg_stdio_src(&cinfo, infile);
    (void) jpeg_read_header(&cinfo, TRUE);
    (void) jpeg_start_decompress(&cinfo);
    width = cinfo.output_width;
    height = cinfo.output_height;

    unsigned char * pDummy = new unsigned char [width*height*3];
    unsigned char * pTest=pDummy;
    if (!pDummy){
        std::stringstream msg;msg<<"Error memory ";msg<<filename;
        throw cpe::io_exception(msg.str(),__FILE__,__FUNCTION__,__LINE__);
    }
    row_stride = width * cinfo.output_components ;
    pJpegBuffer = (*cinfo.mem->alloc_sarray)
            ((j_common_ptr) &cinfo, JPOOL_IMAGE, row_stride, 1);

    while (cinfo.output_scanline < cinfo.output_height) {
        (void) jpeg_read_scanlines(&cinfo, pJpegBuffer, 1);
        for (int x=0;x<width;x++) {
            r = pJpegBuffer[0][cinfo.output_components*x];
            if (cinfo.output_components>2)
            {
                g = pJpegBuffer[0][cinfo.output_components*x+1];
                b = pJpegBuffer[0][cinfo.output_components*x+2];
            } else {
                g = r;
                b = r;
            }
            *(pDummy++) = b;
            *(pDummy++) = g;
            *(pDummy++) = r;
        }
    }
    fclose(infile);
    (void) jpeg_finish_decompress(&cinfo);
    jpeg_destroy_decompress(&cinfo);

    std::vector<color> raw(width*height);
    for(int k=0;k<width*height;++k)
        raw[k]=color(pTest[3*k+2],pTest[3*k+1],pTest[3*k+0]);
    image im(width,height,raw);

    delete pTest;
    pTest=NULL;

    return im;
}



//from http://www.andrewewhite.net/wordpress/2008/09/02/very-simple-jpeg-writer-in-c-c/
void io_jpeg::write(const image& im,const std::string& filename) const
{


    std::vector<octet> raw=im.to_raw_data_rgb();


    FILE* outfile = fopen(filename.c_str(), "wb");
    if (!outfile)
    {
        std::stringstream msg;msg<<"Cannot open file "<<filename;
        throw cpe::io_exception(msg.str(),__FILE__,__FUNCTION__,__LINE__);
    }

    struct jpeg_compress_struct cinfo;
    struct jpeg_error_mgr jerr;

    /* this is a pointer to one row of image data */
    JSAMPROW row_pointer[1];
    cinfo.err = jpeg_std_error( &jerr );
    jpeg_create_compress(&cinfo);
    jpeg_stdio_dest(&cinfo, outfile);

    /* Setting the parameters of the output file here */
    cinfo.image_width = im.Nx();
    cinfo.image_height = im.Ny();
    cinfo.input_components = 3;
    cinfo.in_color_space = JCS_RGB;



    /* default compression parameters, we shouldn't be worried about these */
    jpeg_set_defaults( &cinfo );

    jpeg_set_quality(&cinfo, 100, TRUE);

    /* Now do the compression .. */
    jpeg_start_compress( &cinfo, TRUE );
    /* like reading a file, this time write one row at a time */
    while( cinfo.next_scanline < cinfo.image_height )
    {
        row_pointer[0] = &raw[ cinfo.next_scanline * cinfo.image_width *  cinfo.input_components];
        jpeg_write_scanlines( &cinfo, row_pointer, 1 );
    }
    /* similar to read file, clean up after we're done compressing */
    jpeg_finish_compress( &cinfo );
    jpeg_destroy_compress( &cinfo );
    fclose( outfile );

}

}
