
#include <io_png.hpp>
#include <io_exception.hpp>

#include <lodepng.h>
#include <iostream>
#include <sstream>

#include <jpeglib.h>

namespace cpe
{

image io_png::read(const std::string& filename) const
{
    std::vector<unsigned char> png;
    std::vector<unsigned char> raw; //the raw pixels
    unsigned width, height;



    //load and decode
    lodepng::load_file(png, filename);
    unsigned error = lodepng::decode(raw, width, height,png);

    //if there's an error:
    if(error)
    {
        std::stringstream msg;msg<<"Error loading ";msg<<filename;
        throw cpe::io_exception(msg.str(),__FILE__,__FUNCTION__,__LINE__);
    }

    image im(width,height);
    im.set_raw_data(raw);

    return im;
}

void io_png::write(const image& im,const std::string& filename) const
{
    std::vector<octet> raw=im.to_raw_data_rgba();

    std::vector<unsigned char> png;
    unsigned error = lodepng::encode(png, raw, im.Nx(), im.Ny());
    if(!error)
        lodepng::save_file(png, filename);
    else
    {
        std::stringstream msg;msg<<"Error saving ";msg<<filename;
        throw cpe::io_exception(msg.str(),__FILE__,__FUNCTION__,__LINE__);
    }
}

}
